def train(input_data_path, model_save_path):
    """
    The function to execute the training.

    :param input_data_path: [str], input directory path where all the training file(s) reside in
    :param model_save_path: [str], directory path to save your model(s)
    """
    # TODO: Write your modeling logic

    # TODO: save the model(s) under 'model_save_path'
